-- create lexer with "alex -g HaskiLexer.x"
{

{-
Part of "Haski the Robot"
examination project for adv. funct. progr. 2004

    Copyright (C) 2004  Lars Otten

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-}

{-

  I M P O R T A N T  ! ! !
----------------------------
after creating the lexer with Alex one has to the replace the
alexScanTokens-function with the following:

alexScanTokens str = go (alexStartPos,'\n',str)
  where go inp@(pos,_,str) =
	  case alexScan inp 0 of
		AlexEOF -> return []
		AlexError (p,c,s) -> failE ("lexical error in line "
		                             ++ getLineNo p ++ ", column " 
		                             ++ getColNo p)
		AlexSkip  inp' len     -> go inp'
		AlexToken inp' len act -> do let x = act pos (take len str)
                                             y <- go inp'
                                             return (x:y)


-}

module HaskiLexer where

import ErrorMonad

}

%wrapper "posn"

$digit = 0-9
$alpha = [a-zA-Z]

tokens :-

  $white+				;
  "--".*				;
  ";"					{ \p s -> T p s TBREAK }
  "="					{ \p s -> T p s TDefine }
  "&>"					{ \p s -> T p s TSeq }
  "("					{ \p s -> T p s TBO } -- bracket open
  ")"					{ \p s -> T p s TBC } -- bracket close
-- normal commands
  "go_forward"				{ \p s -> T p s TGo_forward }
  "turn_right"				{ \p s -> T p s TTurn_right }
  "take_item"				{ \p s -> T p s TTake_item }
  "drop_item"				{ \p s -> T p s TDrop_item }
  "do_nothing"				{ \p s -> T p s TDo_nothing }
-- conditionals
  "While"				{ \p s -> T p s TWhile }
  "IfThen"				{ \p s -> T p s TIfThen }
  "IfThenElse"				{ \p s -> T p s TIfThenElse }
  "DoWhile"				{ \p s -> T p s TDoWhile }
-- conditions
  ":&"					{ \p s -> T p s TAND }
  ":|"					{ \p s -> T p s TOR }
  "Not"					{ \p s -> T p s TNOT }
  "front_free"				{ \p s -> T p s TFront_free }
  "right_free"				{ \p s -> T p s TRight_free }
  "left_free"				{ \p s -> T p s TLeft_free }
  "is_carrying"				{ \p s -> T p s TIs_carrying }
  "field_has_item"			{ \p s -> T p s TField_has_item }
  "facing_up"				{ \p s -> T p s TFacing_up }
  "facing_right"			{ \p s -> T p s TFacing_right }
  "facing_down"				{ \p s -> T p s TFacing_down }
  "facing_left"				{ \p s -> T p s TFacing_left }
-- variable names (for functions)
  $alpha [$alpha $digit \_ \-]*		{ \p s -> T p s (TName s) }

{

data TValue
  = TBREAK
  | TDefine
  | TSeq
  | TBO
  | TBC
-- normal commands
  | TGo_forward
  | TTurn_right
  | TTake_item
  | TDrop_item
  | TDo_nothing
-- conditionals
  | TWhile
  | TDoWhile
  | TIfThen
  | TIfThenElse
-- conditions
  | TAND
  | TOR
  | TNOT
  | TFront_free
  | TRight_free
  | TLeft_free
  | TIs_carrying
  | TField_has_item
  | TFacing_up
  | TFacing_right
  | TFacing_down
  | TFacing_left
-- variable names (for functions)
  | TName String
       deriving (Eq,Show)

-- saves the position info, the actual string
-- and the parsed Token
data Token = T AlexPosn String TValue

-- gets the line/column number from a AlexPn-object
getLineNo (AlexPn _ l _) = show l
getColNo (AlexPn _ _ c) = show c

-- main function
lexer s = alexScanTokens s


}
